@@ -31,14 +31,16 @@ import logging
 import datetime
 import uuid
 import itertools
+import unicodedata
 
 from collections import defaultdict
 from typing import TYPE_CHECKING, Optional, Union, Awaitable, Any
 
 from .errors import XMPPError, PartyError
 from .message import FriendMessage, PartyMessage
-from .party import Party, PartyInvitation, PartyJoinConfirmation
+from .party import Party, ReceivedPartyInvitation, PartyJoinConfirmation
 from .presence import Presence
+from .enums import AwayStatus
 
 if TYPE_CHECKING:
     from .client import Client
@@ -46,6 +48,10 @@ if TYPE_CHECKING:
 log = logging.getLogger(__name__)
 
 
+def is_RandALCat(c: str) -> bool:
+    return unicodedata.bidirectional(c) in ('R', 'AL')
+
+
 class EventContext:
 
     __slots__ = ('client', 'body', 'party', 'created_at')
@@ -54,7 +60,7 @@ class EventContext:
         self.client = client
         self.body = body
 
-        self.party = self.client.user.party
+        self.party = self.client.party
         self.created_at = datetime.datetime.utcnow()
 
 
@@ -114,9 +120,19 @@ class XMPPClient:
         self._task = None
         self.muc_room = None
 
-    def jid(self, id: str) -> aioxmpp.JID:
-        return aioxmpp.JID.fromstr('{}@{}'.format(id,
-                                                  self.client.service_host))
+    def jid(self, user_id: str) -> aioxmpp.JID:
+        return aioxmpp.JID.fromstr('{}@{}'.format(
+            user_id,
+            self.client.service_host
+        ))
+
+    def _remove_illegal_characters(self, chars: str) -> str:
+        for c in chars:
+            if is_RandALCat(c):
+                chars = chars.replace(c, '')
+            if ord(c) in (0,):
+                chars = chars.replace(c, '')
+        return chars
 
     def _create_invite(self, from_id: str, data: dict) -> dict:
         sent_at = self.client.from_iso(data['sent'])
@@ -274,7 +290,6 @@ class XMPPClient:
     @dispatcher.event('com.epicgames.social.party.notification.v0.PING')
     async def event_ping_received(self, ctx: EventContext) -> None:
         body = ctx.body
-
         pinger = body['pinger_id']
         try:
             data = (await self.client.http.party_lookup_ping(pinger))[0]
@@ -308,14 +323,24 @@ class XMPPClient:
         new_party = Party(self.client, data)
         await new_party._update_members(members=data['members'])
 
-        invitation = PartyInvitation(self.client, new_party, net_cl, invite)
+        invitation = ReceivedPartyInvitation(
+            self.client,
+            new_party,
+            net_cl,
+            invite
+        )
         self.client.dispatch_event('party_invite', invitation)
 
     @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_JOINED')  # noqa
     async def event_party_member_joined(self,
                                         ctx: EventContext) -> None:
         body = ctx.body
-        party = ctx.party
+
+        user_id = body.get('account_id')
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
 
         if party is None:
             return
@@ -323,7 +348,7 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(body.get('account_id'))
+        member = party.members.get(user_id)
         if member is None:
             member = party._create_member(body)
 
@@ -331,13 +356,14 @@ class XMPPClient:
                 party._create_clientmember(body)
 
         if party.me is not None:
-            asyncio.ensure_future(party.me.patch(), loop=self.client.loop)
+            party.me.do_on_member_join_patch()
 
-        if party.me and party.me.leader:
-            part = party.meta.refresh_squad_assignments()
-            fut = asyncio.ensure_future(party.patch(updated={
-                'RawSquadAssignments_j': part
-            }), loop=self.client.loop)
+        yielding = party.me._default_config.yield_leadership
+        if party.me and party.me.leader and not yielding:
+            fut = asyncio.ensure_future(
+                party.refresh_squad_assignments(),
+                loop=self.client.loop
+            )
 
         try:
             if member.id == self.client.user.id:
@@ -362,7 +388,12 @@ class XMPPClient:
     @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_LEFT')
     async def event_party_member_left(self, ctx: EventContext) -> None:
         body = ctx.body
-        party = ctx.party
+
+        user_id = body.get('account_id')
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
 
         if party is None:
             return
@@ -370,17 +401,26 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(body.get('account_id'))
+        member = party.members.get(user_id)
         if member is None:
             return
 
         party._remove_member(member.id)
+
+        if party.me and party.me.leader and member.id != party.me.id:
+            await party.refresh_squad_assignments()
+
         self.client.dispatch_event('party_member_leave', member)
 
     @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_EXPIRED')  # noqa
     async def event_party_member_expired(self, ctx: EventContext) -> None:
         body = ctx.body
-        party = ctx.party
+
+        user_id = body.get('account_id')
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
 
         if party is None:
             return
@@ -388,22 +428,30 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(body.get('account_id'))
+        member = party.members.get(user_id)
         if member is None:
             return
 
+        party._remove_member(member.id)
+
+        if party.me and party.me.leader and member.id != party.me.id:
+            await party.refresh_squad_assignments()
+
         if member.id == self.client.user.id:
             p = await self.client._create_party()
+            self.client.party = p
 
-            self.client.user.set_party(p)
-
-        party._remove_member(member.id)
         self.client.dispatch_event('party_member_expire', member)
 
     @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_KICKED')  # noqa
     async def event_party_member_kicked(self, ctx: EventContext) -> None:
         body = ctx.body
-        party = ctx.party
+
+        user_id = body.get('account_id')
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
 
         if party is None:
             return
@@ -411,23 +459,32 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(body.get('account_id'))
+        member = party.members.get(user_id)
         if member is None:
             return
+
         party._remove_member(member.id)
 
+        if party.me and party.me.leader and member.id != party.me.id:
+            await party.refresh_squad_assignments()
+
         if member.id == self.client.user.id:
             await self.leave_muc()
             p = await self.client._create_party()
 
-            self.client.user.set_party(p)
+            self.client.party = p
 
         self.client.dispatch_event('party_member_kick', member)
 
     @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_DISCONNECTED')  # noqa
     async def event_party_member_disconnected(self, ctx: EventContext) -> None:
         body = ctx.body
-        party = ctx.party
+
+        user_id = body.get('account_id')
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
 
         if party is None:
             return
@@ -435,11 +492,15 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(body.get('account_id'))
+        member = party.members.get(user_id)
         if member is None:
             return
 
         party._remove_member(member.id)
+
+        if party.me and party.me.leader and member.id != party.me.id:
+            await party.refresh_squad_assignments()
+
         self.client.dispatch_event('party_member_disconnect', member)
 
     @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_NEW_CAPTAIN')  # noqa
@@ -447,13 +508,19 @@ class XMPPClient:
         body = ctx.body
         party = ctx.party
 
+        user_id = body.get('account_id')
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
+
         if party is None:
             return
 
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(body.get('account_id'))
+        member = party.members.get(user_id)
         if member is None:
             return
 
@@ -463,7 +530,7 @@ class XMPPClient:
 
         member.update_role('CAPTAIN')
         if member.id == self.client.user.id:
-            self.client.user.party.me.update_role('CAPTAIN')
+            self.client.party.me.update_role('CAPTAIN')
 
         party.update_presence()
         self.client.dispatch_event('party_member_promote', old_leader, member)
@@ -471,7 +538,12 @@ class XMPPClient:
     @dispatcher.event('com.epicgames.social.party.notification.v0.PARTY_UPDATED')  # noqa
     async def event_party_updated(self, ctx: EventContext) -> None:
         body = ctx.body
-        party = ctx.party
+
+        user_id = body.get('account_id')
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
 
         if party is None:
             return
@@ -500,7 +572,12 @@ class XMPPClient:
     async def event_party_member_state_updated(self,
                                                ctx: EventContext) -> None:
         body = ctx.body
-        party = ctx.party
+
+        user_id = body.get('account_id')
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
 
         if party is None:
             return
@@ -508,10 +585,10 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(body.get('account_id'))
+        member = party.members.get(user_id)
         if member is None:
             def check(m):
-                return m.id == body.get('account_id')
+                return m.id == user_id
 
             try:
                 member = await self.client.wait_for(
@@ -520,15 +597,15 @@ class XMPPClient:
                     timeout=3
                 )
             except asyncio.TimeoutError:
-                if body.get('account_id') == self.client.user.id:
+                if user_id == self.client.user.id:
                     await party._leave()
                     p = await self.client._create_party()
-                    self.client.user.set_party(p)
+                    self.client.party = p
                 return
 
         check = ('ready', 'input', 'assisted_challenge', 'outfit', 'backpack',
                  'pet', 'pickaxe', 'contrail', 'emote', 'emoji', 'banner',
-                 'battlepass_info')
+                 'battlepass_info', 'in_match', 'match_players_left')
         pre_values = {k: getattr(member, k) for k in check}
 
         check_variants = ('outfit_variants', 'backpack_variants',
@@ -536,6 +613,31 @@ class XMPPClient:
         pre_variants_values = {k: getattr(member, k) for k in check_variants}
 
         member.update(body)
+
+        if party._default_config.team_change_allowed or not party.me.leader:
+            req_j = body['member_state_updated'].get(
+                'MemberSquadAssignmentRequest_j'
+            )
+            if req_j is not None:
+                req = json.loads(req_j)['MemberSquadAssignmentRequest']
+                version = req['version']
+                if version != member._assignment_version:
+                    member._assignment_version = version
+
+                    new_positions = {
+                        member.id: req['targetAbsoluteIdx'],
+                        req['swapTargetMemberId']: req['startingAbsoluteIdx']
+                    }
+                    if party.me.leader:
+                        await party.refresh_squad_assignments(
+                            new_positions=new_positions
+                        )
+
+                    self.client.dispatch_event(
+                        'party_member_team_swap',
+                        *[party.members[k] for k in new_positions]
+                    )
+
         self.client.dispatch_event('party_member_update', member)
 
         for key, pre_value in pre_values.items():
@@ -568,7 +670,12 @@ class XMPPClient:
                                                       ctx: EventContext
                                                       ) -> None:
         body = ctx.body
-        party = ctx.party
+
+        user_id = body.get('account_id')
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
 
         if party is None:
             return
@@ -576,7 +683,6 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        user_id = body['account_id']
         user = self.client.get_user(user_id)
         if user is None:
             user = await self.client.fetch_profile(user_id)
@@ -600,15 +706,17 @@ class XMPPClient:
         if '-' in user_id:
             return
 
-        if presence.type_ not in (aioxmpp.PresenceType.AVAILABLE,
-                                  aioxmpp.PresenceType.UNAVAILABLE):
+        if not presence.type_.is_presence_state:
             return
 
         try:
             data = json.loads(presence.status.any())
-            if (data.get('Status', '') == ''
-                    or 'bIsPlaying' not in data
-                    or not isinstance(data.get('Properties', {}), dict)):
+
+            # We do this to filter out kairos from launcher presences
+            ch = data.get('bIsEmbedded', False) or data.get('Status', '') != ''
+
+            is_dict = isinstance(data.get('Properties', {}), dict)
+            if (not ch or 'bIsPlaying' not in data or not is_dict):
                 return
         except ValueError:
             return
@@ -628,11 +736,19 @@ class XMPPClient:
         if not presence.from_.is_bare:
             platform = presence.from_.resource.split(':')[2]
 
+        is_available = presence.type_ is aioxmpp.PresenceType.AVAILABLE
+
+        try:
+            away = AwayStatus(presence.show)
+        except ValueError:
+            away = AwayStatus.ONLINE
+
         _pres = Presence(
             self.client,
             user_id,
             platform,
-            presence.type_ is aioxmpp.PresenceType.AVAILABLE,
+            is_available,
+            away,
             data
         )
 
@@ -750,7 +866,7 @@ class XMPPClient:
 
     async def send_presence_on_start(self) -> None:
         await self.client.wait_until_ready()
-        await self.send_presence(status=self.client.user.party.last_raw_status)
+        await self.send_presence(status=self.client.party.last_raw_status)
 
     async def close(self) -> None:
         log.debug('Attempting to close xmpp client')
@@ -784,7 +900,7 @@ class XMPPClient:
             return
 
         user_id = member.direct_jid.localpart
-        party = self.client.user.party
+        party = self.client.party
 
         if (user_id == self.client.user.id or member.nick is None
                 or user_id not in party.members):
@@ -809,7 +925,7 @@ class XMPPClient:
                      muc_reason: str,
                      **kwargs: Any) -> None:
         if muc_leave_mode is aioxmpp.muc.LeaveMode.BANNED:
-            mem = self.client.user.party.members[member.direct_jid.localpart]
+            mem = self.client.party.members[member.direct_jid.localpart]
             self.client.dispatch_event('party_member_chatban',
                                        mem,
                                        muc_reason)
@@ -818,12 +934,16 @@ class XMPPClient:
         muc_jid = aioxmpp.JID.fromstr(
             'Party-{}@muc.prod.ol.epicgames.com'.format(party_id)
         )
-        nick = '{0.display_name}:{0.id}:{1}'.format(
-            self.client.user,
+        nick = '{0}:{0}:{1}'.format(
+            self._remove_illegal_characters(self.client.user.display_name),
+            self.client.user.id,
             self.xmpp_client.local_jid.resource
         )
 
-        room, fut = self.muc_service.join(muc_jid, nick)
+        room, fut = self.muc_service.join(
+            muc_jid,
+            nick
+        )
 
         room.on_message.connect(self.muc_on_message)
         room.on_join.connect(self.muc_on_member_join)
